@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2012 Petri Lehtinen <petri@digip.org>
+ * Copyright (c) 2009-2013 Petri Lehtinen <petri@digip.org>
  *
  * Jansson is free software; you can redistribute it and/or modify
  * it under the terms of the MIT license. See LICENSE for details.
@@ -11,7 +11,7 @@
 static void test_misc(void)
 {
     json_t *array, *five, *seven, *value;
-    int i;
+    size_t i;
 
     array = json_array();
     five = json_integer(5);
@@ -206,6 +206,7 @@ static void test_insert(void)
 static void test_remove(void)
 {
     json_t *array, *five, *seven;
+    int i;
 
     array = json_array();
     five = json_integer(5);
@@ -253,6 +254,19 @@ static void test_remove(void)
        json_array_get(array, 2) != seven)
         fail("remove works incorrectly");
 
+    json_decref(array);
+
+    array = json_array();
+    for(i = 0; i < 4; i++) {
+        json_array_append(array, five);
+        json_array_append(array, seven);
+    }
+    if(json_array_size(array) != 8)
+        fail("unable to append 8 items to array");
+
+    /* Remove an element from a "full" array. */
+    json_array_remove(array, 5);
+
     json_decref(five);
     json_decref(seven);
     json_decref(array);
@@ -386,6 +400,25 @@ static void test_circular()
     json_decref(array1);
 }
 
+static void test_array_foreach()
+{
+    size_t index;
+    json_t *array1, *array2, *value;
+
+    array1 = json_pack("[sisisi]", "foo", 1, "bar", 2, "baz", 3);
+    array2 = json_array();
+
+    json_array_foreach(array1, index, value) {
+        json_array_append(array2, value);
+    }
+    
+    if(!json_equal(array1, array2))
+        fail("json_array_foreach failed to iterate all elements");
+
+    json_decref(array1);
+    json_decref(array2);
+}
+
 
 static void run_tests()
 {
@@ -395,4 +428,5 @@ static void run_tests()
     test_clear();
     test_extend();
     test_circular();
+    test_array_foreach();
 }
